//
//////////////////////////////////////////////////////////////////////

#if !defined(HTMatrix_Header_File)
#define HTMatrix_Header_File

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


#define TMatrix(type) HTMatrix<type>

#define TMatrixdeclare(type,LOCATION) template class LOCATION HTMatrix<type>;

//#include "Epsilon.hpp"
inline char Epsilon(char a){ return (char)(1); };
inline int Epsilon(int a){ return 1; };
inline unsigned Epsilon(unsigned a){ return 1u; };
inline long Epsilon(long a){ return 1L; };
inline float Epsilon(float a){ return (float)(1e-5); };
inline double Epsilon(double a){ return 1e-15; };

TEMPLATED class HTMatrix
{
public:
	type				m_HTMatrixA,m_HTMatrixB,m_HTMatrixC,m_HTMatrixD;//a,b,c,d,;
	type				m_HTMatrixTx,m_HTMatrixTy;//tx,ty;
	//-------------------------------------------------
	HTMatrix<type> operator*(const HTMatrix<type> &arg)
	{
		HTMatrix<type> tmp;
		tmp.Init(
			m_HTMatrixA*arg.m_HTMatrixA + m_HTMatrixB*arg.m_HTMatrixC,
			m_HTMatrixA*arg.m_HTMatrixB + m_HTMatrixB*arg.m_HTMatrixD,
			m_HTMatrixC*arg.m_HTMatrixA + m_HTMatrixD*arg.m_HTMatrixC,
			m_HTMatrixC*arg.m_HTMatrixB + m_HTMatrixD*arg.m_HTMatrixD,
			arg.m_HTMatrixTx + m_HTMatrixTx*arg.m_HTMatrixA + m_HTMatrixTy*arg.m_HTMatrixC,
			arg.m_HTMatrixTy + m_HTMatrixTx*arg.m_HTMatrixB + m_HTMatrixTy*arg.m_HTMatrixD
		);
		return tmp;
	};
	//-------------------------------------------------
	HTMatrix<type> &operator*=(const HTMatrix<type> &arg)
	{
		type tmp;
		tmp = m_HTMatrixA*arg.m_HTMatrixA+m_HTMatrixB*arg.m_HTMatrixC;
		m_HTMatrixB *= arg.m_HTMatrixD;
		m_HTMatrixB += m_HTMatrixA*arg.m_HTMatrixB;
		m_HTMatrixA = tmp;
		tmp = m_HTMatrixC*arg.m_HTMatrixA + m_HTMatrixD*arg.m_HTMatrixC;
		m_HTMatrixD *= arg.m_HTMatrixD; m_HTMatrixD += m_HTMatrixC*arg.m_HTMatrixB;
		m_HTMatrixC = tmp;
		tmp = arg.m_HTMatrixTx + m_HTMatrixTx*arg.m_HTMatrixA + m_HTMatrixTy*arg.m_HTMatrixC;
		m_HTMatrixTy *= arg.m_HTMatrixD;
		m_HTMatrixTy += m_HTMatrixTx*arg.m_HTMatrixB + arg.m_HTMatrixTy;
		m_HTMatrixTx = tmp;
		return *this;
	};
	//-------------------------------------------------
	int operator==(const HTMatrix<type> &arg) const
	{
		return memcmp(this,&arg,sizeof(*this))==EQUAL;
	};
	//-------------------------------------------------
	int operator!=(const HTMatrix<type> &arg) const
	{
		return memcmp(this,&arg,sizeof(*this))!=EQUAL;
	};
	//-------------------------------------------------
	// t֐iɖ߂}gXNW쐬j
	HTMatrix<type> Inv(void) const
	{
		HTMatrix<type> IM;
		type det = m_HTMatrixA*m_HTMatrixD - m_HTMatrixB*m_HTMatrixC;
		if(fabs(det)<=Epsilon(det)) return *this;
		IM.Init(m_HTMatrixD/det,-m_HTMatrixB/det,-m_HTMatrixC/det,m_HTMatrixA/det,
			(m_HTMatrixTy*m_HTMatrixC-m_HTMatrixTx*m_HTMatrixD)/det,
			(m_HTMatrixTx*m_HTMatrixB-m_HTMatrixTy*m_HTMatrixA)/det);
		return(IM);
	};
	//-------------------------------------------------
	// ]
	HTMatrix<type> &Rotate(type alpha)
	{
		type c=cos(alpha),s=sin(alpha);
		HTMatrix<type> tmp(c,s,-s,c,0.0,0.0);
		return((*this)*=tmp);
	};
	//-------------------------------------------------
	// ړ
	HTMatrix<type> &Translate(type transx, type transy)
	{
		m_HTMatrixTx += transx;
		m_HTMatrixTy += transy;
		return *this;
	};
	//-------------------------------------------------
	// kEg
	HTMatrix<type> &Scale(type sx, type sy)
	{
		m_HTMatrixA *= sx;
		m_HTMatrixB *= sy;
		m_HTMatrixC *= sx;
		m_HTMatrixD *= sy;
		return *this;
	};
	//-------------------------------------------------
	// Α
	HTMatrix<type> &Skew(double alpha)
	{
		type s = sin(alpha);
		m_HTMatrixB += m_HTMatrixA*s;
		m_HTMatrixD += m_HTMatrixC*s;
		return *this;
	};
	//-------------------------------------------------
	// QRect񂩂̈ړEkEg
	void BoxTransformation(type x0, type y0, type x1, type y1,type tx0, type ty0, type tx1, type ty1)
	{
		type Dx = x1-x0;
		type Dy = y1-y0;
		m_HTMatrixA = (tx1-tx0)/Dx;
		m_HTMatrixD = (ty1-ty0)/Dy;
		m_HTMatrixB = m_HTMatrixC = 0.0;
		m_HTMatrixTx = (tx0*x1-tx1*x0)/Dx;
		m_HTMatrixTy = (ty0*y1-ty1*y0)/Dy;
	};
	//-------------------------------------------------
	// ϊlNA
	void IdentMatrix(void)
	{
		m_HTMatrixA=m_HTMatrixD=1;
		m_HTMatrixB=m_HTMatrixC=m_HTMatrixTx=m_HTMatrixTy=0;
	};
	//-------------------------------------------------
	// ϊlNAĂ邩`FbN
	int  IsIdentMatrix() const
	{
		HTMatrix<type> I;
		return *this==I;
	};
	//-------------------------------------------------
	// ]AY[邩`FbN
	int  IsTranslateOnly() const
	{
		return ((int)((m_HTMatrixA-1.)*1000.)==0 && m_HTMatrixB==0 && m_HTMatrixC==0 
			&& (int)((m_HTMatrixD-1.)*1000.)==0);
	};
	//-------------------------------------------------
	// ϊ̎s
	void Transform(type &x, type &y) const
	{
		type tmp;
		if( !IsTranslateOnly() )
		{
			tmp = m_HTMatrixA*x + m_HTMatrixC*y + m_HTMatrixTx;
			y = m_HTMatrixB*x + m_HTMatrixD*y + m_HTMatrixTy;
			x = tmp;
		}
		else
		{
			x += m_HTMatrixTx; y += m_HTMatrixTy;
		}
	};
	//-------------------------------------------------
	// ϊ̎s-(Translate)ړȂ
	void dTransform(type &dx, type &dy) const
	{
		type tmp = m_HTMatrixA*dx + m_HTMatrixC*dy;
		dy = m_HTMatrixB*dx + m_HTMatrixD*dy;
		dx = tmp;
	};
	//-------------------------------------------------
	// X = naX + ncY + ntx;
	// Y = nbX + ndY + nty;
	void Init(type na=1, type nb=0, type nc=0, type nd=1, type ntx=0, type nty=0)
	{
		m_HTMatrixA=na; m_HTMatrixB=nb; m_HTMatrixC=nc; 
		m_HTMatrixD=nd; m_HTMatrixTx=ntx; m_HTMatrixTy=nty;
	};
	//-------------------------------------------------
	// X = naX + ncY + ntx;
	// Y = nbX + ndY + nty;
	HTMatrix<type>(type na=1, type nb=0, type nc=0, type nd=1, type ntx=0, type nty=0)
	{
		m_HTMatrixA=na; m_HTMatrixB=nb; m_HTMatrixC=nc; 
		m_HTMatrixD=nd; m_HTMatrixTx=ntx; m_HTMatrixTy=nty;
	};

	//-------------------------------------------------
	// p
	//-------------------------------------------------
	void Transform(HBBox<type> &B) const
	{
		HBBox<type> Tmp;
		HPoint<type> P;
		P.Fill(B.m_HBBoxBx.m_HRangeMin,B.m_HBBoxBy.m_HRangeMin);
		Transform(P.m_HPointX,P.m_HPointY); Tmp = P;

		P.Fill(B.m_HBBoxBx.m_HRangeMax,B.m_HBBoxBy.m_HRangeMin);
		Transform(P.m_HPointX,P.m_HPointY); Tmp += P;

		P.Fill(B.m_HBBoxBx.m_HRangeMax,B.m_HBBoxBy.m_HRangeMax);
		Transform(P.m_HPointX,P.m_HPointY); Tmp += P;

		P.Fill(B.m_HBBoxBx.m_HRangeMin,B.m_HBBoxBy.m_HRangeMax);
		Transform(P.m_HPointX,P.m_HPointY); Tmp += P;
		B = Tmp;
	};
	//-------------------------------------------------
	void Transform(CPoint &P) const
	{
		type x = P.x;
		type y = P.y;
		Transform(x,y);
		P.x = (int)x;
		P.y = (int)y;
	};
	//-------------------------------------------------
	HBBox<type> Transform(const CRect &R) const
	{
		HBBox<type> B;
		B.m_HBBoxBx = R.left;
		B.m_HBBoxBx += R.right;

		B.m_HBBoxBy = R.top;
		B.m_HBBoxBy += R.bottom;
		Transform(B);
		return B;
	};
	//-------------------------------------------------
	void Transform(CRect &R) const
	{
		Transform(R.TopLeft());
		Transform(R.BottomRight());
	};            
	//-------------------------------------------------
	CPoint Transform(const HPoint<type> &P,CPoint &pnt) const
	{
		type x = P.m_HPointX;
		type y = P.m_HPointY;
		Transform(x,y);
		pnt = CPoint(Round(x),Round(y));
		return pnt;
	};
	//-------------------------------------------------
	const HPoint<int> Transform(HPoint<type> P) const
	{
		HPoint<int> result;
		Transform(P.m_HPointX,P.m_HPointY);
		result.m_HPointX = Round(P.m_HPointX);
		result.m_HPointY = Round(P.m_HPointY);
		return result;
	}
	//-------------------------------------------------
	void Transform(HBBox<type> &B, HBBox<int> &R) const
	{ 
		R =  Transform(B.LB());
		R += Transform(B.RB());
		R += Transform(B.RT());
		R += Transform(B.LT());
	};
	void Transform(HBBox<type> &B, CRect &R) const
	{  
		HBBox<int> tmp;
		Transform(B,tmp);
		R.left   = Round(tmp.m_HBBoxBx.m_HRangeMin);
		R.top    = Round(tmp.m_HBBoxBy.m_HRangeMin);
		R.right  = Round(tmp.m_HBBoxBx.m_HRangeMax);
		R.bottom = Round(tmp.m_HBBoxBy.m_HRangeMax);  
	};
};

#define TMatriximplement(type)

#endif // !defined(HTMatrix_Header_File)


