#pragma once
#include <vector>
#include <list>
#include "SGFdData.h"
#include "SGString.h"

/// @brief タイマー情報構造体
typedef struct {
	unsigned long long mTime; ///< @brief 時間(ミリ秒)
	unsigned int mId;         ///< @brief 識別子
	SGString mName;           ///< @brief ダイアログ名
	unsigned mInterval;       ///< @brief インターバル（シングルはゼロ）
} SGTimerData;

typedef std::list<SGTimerData> SGTimerList;        ///< @brief タイマー情報構造体リスト
typedef SGTimerList::iterator SGTimerListiterator; ///< @brief タイマー情報構造体リストのイテレーター

typedef struct timeval TimeVal; ///< @brief timeval 構造体と等価

//--------------------------------------------------------------------
/**
@brief PlusGアプリケーションの基本クラス

リソースエディタを使用してアプリケーションを作成した場合、自動的にこのクラスが継承されます。
*/
class SGApplication
{
public:
	SGApplication();
	virtual ~SGApplication();

//  SGString GetVersion();
	virtual int AppMain();

	virtual void ApplicationTimer(TimeVal time,unsigned int id);
	virtual void DialogTimer(TimeVal time,unsigned int id,SGString name);

	int AddEventFd(SGFdData* fdd);
	void RemoveEventFd(SGFdData* fdd);
	virtual SGFdDataList* GetFdDataList();
	
	void SetApplicationTimerInterval(unsigned long interval,unsigned int id=100);
	void SetApplicationSingleTimerInterval(unsigned long interval,unsigned int id=200);
	
	void SetTimerData(unsigned long interval,unsigned int id,SGString name,int cont);
	int Initialize(int argc, char **argv);

	void SetDateTime(int year,int mon,int day,int hour=-1,int min=-1,int sec=-1);
	void ResetTimer();

	void WakeUp();

private:
	unsigned long long BaseTimer(TimeVal tn);

private:
	SGTimerList mTimerList;     ///< @brief シングルタイマーリスト

	SGFdDataList mFdList; ///< @brief ファイルディスクリプタリスト

	SGSemaphore mTimerListLock; ///< @brief シングルタイマーリストのセマフォ
};
